@@ -10,31 +10,25 @@
 #' out_of_date("data/processed_data.Rds", "data/raw_data.Rds")
 #' }
 out_of_date <- function(targets, dependencies, packages = NULL) {
-  stopifnot(is.character(targets))
-  stopifnot(is.character(dependencies))
+  stopifnot_class(targets, "character")
+  if (!is.null(dependencies)) stopifnot_class(dependencies, "character")
 
   outdated <- any(unlist(lapply(targets, function(fp_x) {
+    if (!file.exists(fp_x)) return(TRUE)
     unlist(lapply(dependencies, function(fp_y) {
       # Target x older than dependency y?
-      if (!file.exists(fp_y)) stop("{fp_y} does not exist", call. = FALSE)
-      if (!file.exists(fp_x)) {
-        return(TRUE)
-      }
+      if (!file.exists(fp_y)) stop('One or more `dependencies` do not exist: ', fp_y, call. = FALSE)
       file.mtime(fp_x) < file.mtime(fp_y)
     }))
   })))
-  if (outdated) {
-    return(outdated)
-  }
+
+  if (outdated) return(outdated)
 
   if (!is.null(packages)) {
-    stopifnot(is.character(packages))
+    stopifnot_class(packages, "character")
     outdated <- outdated | any(unlist(lapply(targets, function(fp_x) {
       unlist(lapply(packages, function(fp_y) {
         # Target x older than package y?
-        if (!file.exists(fp_x)) {
-          return(TRUE)
-        }
         file.mtime(fp_x) < package_datetime(packages)
       }))
     })))
