@@ -15,6 +15,22 @@ source2 <- tempfile(fileext=".R")
 file.copy(system.file("tests", "source2.R", package = "makepipe"), source2)
 
 
+# Test Pipeline -----------------------------------------------------------
+#' Define a Pipeline subclass to expose private fields for testing:
+
+PipelineTest <- R6::R6Class(
+  inherit = Pipeline,
+  public = list(
+    get_nodes = function() {
+      private$nodes
+    },
+    is_outdated = function(node) {
+      self$refresh()
+      node %in% private$edges[private$edges$.outdated, "to"]
+    }
+  )
+)
+
 # Functions --------------------------------------------------------------------
 order_filetimes <- function(...) {
   all_files <- list(...)
@@ -26,17 +42,15 @@ order_filetimes <- function(...) {
 }
 
 expect_outofdate <- function(node_id, pipeline = get_pipeline()) {
-  nodes <- pipeline$nodes
   expect(
-    all(nodes[nodes$id %in% node_id, "group"] %in% "Out-of-date"),
+    all(pipeline$is_outdated(node_id)),
     "`target` should be out of date but isn't"
   )
 }
 
 expect_uptodate <- function(node_id, pipeline = get_pipeline()) {
-  nodes <- pipeline$nodes
   expect(
-    all(nodes[nodes$id %in% node_id, "group"] %in% "Up-to-date"),
+    all(!pipeline$is_outdated(node_id)),
     "`target` should be up to date but isn't"
   )
 }
@@ -45,28 +59,28 @@ expect_uptodate <- function(node_id, pipeline = get_pipeline()) {
 
 ## make_with_source ------------------------------------------------------------
 test_that("targets rebuilt if older than source", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(dependency, target1, source1)
-  make_with_source(source1, target1, dependency, quiet = TRUE)
-  expect_outofdate(target1)
+  res <- make_with_source(source1, target1, dependency, quiet = TRUE)
+  expect_true(res$executed)
 })
 
 test_that("targets rebuilt if older than dependency", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(source1, target1, dependency)
-  make_with_source(source1, target1, dependency, quiet = TRUE)
-  expect_outofdate(target1)
+  res <- make_with_source(source1, target1, dependency, quiet = TRUE)
+  expect_true(res$executed)
 })
 
 test_that("targets rebuilt if older than dependency and source", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(target1, source1, dependency)
-  make_with_source(source1, target1, dependency, quiet = TRUE)
-  expect_outofdate(target1)
+  res <- make_with_source(source1, target1, dependency, quiet = TRUE)
+  expect_true(res$executed)
 })
 
 test_that("out-of-dateness is passed along", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(target1, source1, dependency, source2, target2)
   make_with_source(source1, target1, dependency, quiet = TRUE)
   expect_outofdate(target1)
@@ -75,15 +89,26 @@ test_that("out-of-dateness is passed along", {
 })
 
 test_that("targets not rebuilt if newer than dependency and source", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(source1, dependency, target1)
-  make_with_source(source1, target1, dependency, quiet = TRUE)
-  expect_uptodate(target1)
+  res <- make_with_source(source1, target1, dependency, quiet = TRUE)
+  expect_false(res$executed)
 })
 
+test_that("targets rebuilt if forced", {
+  set_pipeline(PipelineTest$new())
+  order_filetimes(source1, dependency, target1)
+  res <- make_with_source(source1, target1, dependency, quiet = TRUE, force = TRUE)
+  expect_true(res$executed)
+
+  set_pipeline(PipelineTest$new())
+  order_filetimes(dependency, target1)
+  res <- make_with_recipe(2+2, target1, dependency, quiet = TRUE, force = TRUE)
+  expect_true(res$executed)
+})
 
 test_that("source never out-of-dated", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(dependency, source1, target1)
   make_with_recipe(source1, target1, dependency, quiet = TRUE)
   expect_uptodate(source1)
@@ -91,29 +116,33 @@ test_that("source never out-of-dated", {
 
 ## make_with_recipe ------------------------------------------------------------
 test_that("targets rebuilt if older than dependency", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(target1, dependency)
-  make_with_recipe({
+  res <- make_with_recipe({
     mt <- read.csv(dependency, check.names = FALSE)
     write.csv(mt, target1, row.names = FALSE)
   }, target1, dependency, quiet = TRUE)
-  expect_outofdate(target1)
+  expect_true(res$executed)
 })
 
 test_that("out-of-dateness is passed along", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
 
-  order_filetimes(target1, source1, dependency, source2, target2)
   make_with_recipe({
     mt <- read.csv(dependency, check.names = FALSE)
     write.csv(mt, target1, row.names = FALSE)
   }, target1, dependency, quiet = TRUE)
-  expect_outofdate(target1)
 
   make_with_recipe({
     mt <- read.csv(target1)
     write.table(mt, target2, sep = "|")
   }, target2, target1, quiet = TRUE)
+
+  # target2 is newer than its dependencies but target1 is older than its dependencies.
+  # So target2 should be out of date since one of its dependencies (target1) is out of date.
+  order_filetimes(target1, source1, dependency, source2, target2)
+
+  expect_outofdate(target1)
   expect_outofdate(target2)
 })
 
@@ -121,7 +150,7 @@ test_that("out-of-dateness is passed along", {
 # show_pipeline ----------------------------------------------------------------
 
 test_that("out-of-dateness is kept up-to-date", {
-  set_pipeline(Pipeline$new())
+  set_pipeline(PipelineTest$new())
   order_filetimes(target1, source1, dependency, source2, target2)
   make_with_source(source1, target1, dependency, quiet = TRUE)
   make_with_source(source2, target2, target1, quiet = TRUE)
@@ -132,23 +161,41 @@ test_that("out-of-dateness is kept up-to-date", {
 })
 
 # Annotations ------------------------------------------------------------------
-set_pipeline(Pipeline$new())
+set_pipeline(PipelineTest$new())
 order_filetimes(dependency, target1, source1)
 make_with_source(source1, target1, dependency, quiet = TRUE)
 pipe <- get_pipeline()
 
+test_that("annotations aren't overwritten by print method", {
+  annotation <- c("one", "two", "three")
+  names(annotation) <- c(dependency, source1, target1)
+  pipe$annotate(labels = annotation, tooltips = annotation)
+
+  print(pipe)
+  expect_identical(sort(pipe$get_nodes()$title), sort(c("one", "two", "three")))
+  expect_identical(sort(pipe$get_nodes()$label), sort(c("one", "two", "three")))
+
+  # Change annotations
+  annotation <- c("1", "2", "3")
+  names(annotation) <- c(dependency, source1, target1)
+  pipe$annotate(labels = annotation, tooltips = annotation)
+
+  print(pipe)
+  expect_identical(sort(pipe$get_nodes()$title), sort(c("1", "2", "3")))
+  expect_identical(sort(pipe$get_nodes()$label), sort(c("1", "2", "3")))
+})
+
+# Duplicates
 test_that("duplicate annotations are disallowed", {
   annotation <- c("one", "two")
   names(annotation) <- rep(dependency, 2)
 
-  expect_error(annotate_pipeline(
-    pipe,
+  expect_error(pipe$annotate(
     tooltips = annotation,
     labels = NULL
   ), regexp = "must not be duplicated")
 
-  expect_error(annotate_pipeline(
-    pipe,
+  expect_error(pipe$annotate(
     labels = annotation,
     tooltips = NULL
   ), regexp = "must not be duplicated")
@@ -156,11 +203,10 @@ test_that("duplicate annotations are disallowed", {
 
 # Non-existent
 test_that("annotations cannot be applied to nodes that don't exist", {
-  expect_error(annotate_pipeline(
-    pipe,
+  expect_error(pipe$annotate(
     tooltips = c("R/aaa.R" = "input"),
     labels = NULL
-  ), regexp = "not nodes in `pipeline`")
+  ), regexp = "not nodes in `Pipeline`")
 })
 
 # Non-character
@@ -168,14 +214,33 @@ test_that("annotations cannot must be character", {
   tooltips <- c(1)
   names(tooltips) <- dependency
 
-  expect_error(annotate_pipeline(
-    pipe,
+  expect_error(pipe$annotate(
     tooltips = tooltips,
     labels = NULL
-  ), regexp = "is.character")
+  ), regexp = "must be of class character")
 })
 
 
+# Clean and build --------------------------------------------------------------
+
+test_that("cleaning triggers rebuild", {
+  set_pipeline(PipelineTest$new())
+
+  make_with_recipe({
+    mt <- read.csv(dependency, check.names = FALSE)
+    write.csv(mt, target1, row.names = FALSE)
+  }, target1, dependency, quiet = TRUE)
+
+  make_with_recipe({
+    mt <- read.csv(target1)
+    write.table(mt, target2, sep = "|")
+  }, target2, target1, quiet = TRUE)
+
+  pipe <- get_pipeline()
+  pipe$clean()
+  expect_snapshot(pipe$build())
+})
+
 # Unlink ------------------------------------------------------------------
 
 unlink(c("dependency", "target1", "target2", "source1", "source2"))
